"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _utils = require("@modern-js/utils");

var _core = require("@babel/core");

var _utils2 = require("./utils");

const isResoureInSrc = (srcDir, resourecPath) => !_utils.path.relative(srcDir, resourecPath).includes('..');

const getImportFileDistPath = (compilerFile, srcDir, importName) => {
  const dir = _utils.path.dirname(compilerFile);

  const compilerFileRelativeLoc = _utils.path.relative(dir, srcDir);

  const importFileRelativeLoc = _utils.path.relative(srcDir, _utils.path.join(dir, importName));

  const inSrc = isResoureInSrc(srcDir, _utils.path.join(dir, importName));

  const importFileDistPath = _utils.path.join(inSrc ? '..' : '../..', compilerFileRelativeLoc, 'styles', importFileRelativeLoc);

  return importFileDistPath;
};

const isStaticFile = file => {
  const tests = [/\.json$/, /\.css$/, /\.less$/, /\.sass$/, /\.scss$/, /\.(jpg|png|jpeg|gif|svg)$/, /\.(jpg|png|jpeg|gif|svg)\?(inline|url)$/];
  return tests.some(regex => regex.test(file));
};

const isStyleFile = file => {
  const tests = [/\.css$/, /\.less$/, /\.sass$/, /\.scss$/];
  return tests.some(regex => regex.test(file));
};

const getReplacePath = (importName, filename, srcDir, importStyle) => {
  if (!filename || !importName) {
    return '';
  }

  if (!isStaticFile(importName)) {
    return '';
  }

  if (!(0, _utils2.isProjectFile)(importName)) {
    return '';
  }

  let realFilepath = getImportFileDistPath(filename, srcDir, importName);

  if (isStyleFile(realFilepath) && importStyle === 'compiled-code') {
    realFilepath = realFilepath.replace(/\.(less|sass|scss)$/, '.css');
  }

  return realFilepath;
};

const importPath = () => ({
  name: 'import-path',
  visitor: {
    ImportDeclaration({
      node
    }, {
      opts,
      file
    }) {
      const {
        source
      } = node;
      const {
        appDirectory,
        importStyle = 'source-code'
      } = opts;
      const srcDir = `${appDirectory}/src`;
      const {
        opts: {
          filename
        }
      } = file;
      const importName = source !== null && source !== void 0 && source.value ? source.value : undefined;
      const replaceValue = getReplacePath(importName, filename, srcDir, importStyle);

      if (replaceValue) {
        node.source.value = replaceValue;
      }
    },

    // dynamic import
    CallExpression({
      node
    }, {
      opts,
      file
    }) {
      const {
        appDirectory,
        importStyle = 'source-code'
      } = opts;
      const srcDir = `${appDirectory}/src`;
      const {
        filename
      } = file.opts;
      const {
        callee,
        arguments: args
      } = node;

      if (callee.type === 'Import' || callee.type === 'Identifier' && callee.name === 'require') {
        const firstArg = args[0];

        if (firstArg.value) {
          const importName = firstArg.value;
          const replaceValue = getReplacePath(importName, filename, srcDir, importStyle);

          if (replaceValue) {
            node.arguments = [_core.types.stringLiteral(replaceValue)];
          }
        }
      }
    }

  }
});

var _default = importPath;
exports.default = _default;